package com.calibre.webserver;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;

import org.jboss.netty.bootstrap.ServerBootstrap;
import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;

import com.calibre.webserver.file.FileManager;
import com.calibre.webserver.net.HTTPPipelineFactory;
import com.calibre.webserver.net.HTTPRequestHandler;

/**
 * Calibre web server is an experimental web server written in Java
 * by Tommo. It used JBoss Netty for networking due to its high performance
 * and good scalability.
 * @author tommo
 *
 */
public class WebServer {
	
	private static WebServer singleton;
	private static Config config;
	private FileManager fileManager;
	private HTTPRequestHandler requestHandler;
	
	private WebServer() {
	}
	
	private void start() {
		log("Launching web server...");
		bind();
		fileManager = new FileManager();
		requestHandler = new HTTPRequestHandler();
	}
	
	private void bind() {
		int num = Runtime.getRuntime().availableProcessors() * 2;
		log("Initializing thread pool of size " + num);
        ServerBootstrap bootstrap = new ServerBootstrap(new NioServerSocketChannelFactory(
                Executors.newFixedThreadPool(num), Executors.newFixedThreadPool(num)));
        bootstrap.setPipelineFactory(new HTTPPipelineFactory());
        bootstrap.bind(new InetSocketAddress("0.0.0.0", config.getPort()));
        log("Webserver bound to port " + config.getPort() + ".");
	}
	
	public static void main(String[] args) {
		int port = args.length > 0 ? Integer.parseInt(args[0]) : 5555;
		singleton = new WebServer();
		config = new Config();
		config.setPort(port);
		singleton.start();
	}
	
	public HTTPRequestHandler getRequestHandler() {
		return requestHandler;
	}
	
	public FileManager getFileManager() {
		return fileManager;
	}

	public static WebServer instance() {
		return singleton;
	}
	
	public static Config config() {
		return config;
	}
	
	public static void log(String msg) {
		System.out.println("WEBSERVER: " + msg);
	}
	
	public static void err(String msg) {
		System.out.println("ERROR: " + msg);
	}
	
	public static void warn(String msg) {
		System.out.println("WARNING: " + msg);
	}

}
