package com.calibre.webserver.file;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Map;

import com.calibre.webserver.WebServer;

/**
 * The web server file manager. The file manager contains methods for checking, fetching, files etc.
 * @author tommo
 *
 */
public class FileManager {
	
	/**
	 * The file cache which contains buffers of files which have been requested.
	 */
	private Map<String, MappedByteBuffer> fileCache = new HashMap<String, MappedByteBuffer>();
	
	/**
	 * Fetches a file.
	 * @param fileName The name of the file.
	 * @return A byte buffer of the file.
	 */
	public FileResponse fetch(String fileName) {
		WebServer.log("Fetching file: " + fileName);
		if (fileCache.containsKey(fileName)) {
			return new FileResponse(getFileType(fileName), fileCache.get(fileName));
		} else {
			if (fileName.contains("../")) {
				WebServer.warn("Request containing directory traversal has been denied.");
				return null;
			}
			try {
				File file = new File(WebServer.config().getFileDir() + fileName);
				FileChannel channel = new FileInputStream(file).getChannel();
				MappedByteBuffer buffer = channel.map(FileChannel.MapMode.READ_ONLY, 0, file.length());
				fileCache.put(fileName, buffer);
				return new FileResponse(getFileType(fileName), buffer);
			} catch (IOException ioe) {
				WebServer.err("File request not found: " + fileName);
			}
		}
		return null;
	}
	
	/**
	 * Gets the file type of the given file.
	 * @param name The name of the file.
	 * @return The {@link FileType} of the file.
	 */
	public FileType getFileType(String name) {
		if (name.contains(".jpeg")) {
			return FileType.JPEG;
		} else if (name.contains(".jpg")) {
			return FileType.JPEG;
		} else if (name.contains(".gif")) {
			return FileType.GIF;
		} else if (name.contains(".png")) {
			return FileType.PNG;
		} else if (name.contains(".html")) {
			return FileType.HTML;
		} else if (name.contains(".html")) {
			return FileType.HTML;
		} else if (name.contains(".txt")) {
			return FileType.TEXT;
		} else {
			return FileType.TEXT;
		}
	}

}
