package com.calibre.webserver.net;

import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.handler.codec.http.DefaultHttpResponse;
import org.jboss.netty.handler.codec.http.HttpMethod;
import org.jboss.netty.handler.codec.http.HttpRequest;
import org.jboss.netty.handler.codec.http.HttpResponse;
import org.jboss.netty.handler.codec.http.HttpResponseStatus;
import org.jboss.netty.handler.codec.http.HttpVersion;
import org.jboss.netty.util.CharsetUtil;

import com.calibre.webserver.Config;
import com.calibre.webserver.WebServer;
import com.calibre.webserver.file.FileResponse;
import com.calibre.webserver.net.response.ResponseBuilder;

/**
 * Handles HTTP requests.
 * 
 * @author Tommo
 * 
 */
public class HTTPRequestHandler {

	/**
	 * Constructs a http request handler.
	 */
	public HTTPRequestHandler() {

	}

	/**
	 * Handles a http request.
	 * 
	 * @param context
	 *            The channel handler context.
	 * @param request
	 *            The {@link HttpRequest} to handle.
	 */
	public void handle(ChannelHandlerContext context, HttpRequest request) {
		if (request.getMethod() != HttpMethod.GET) {
			sendResponse(context, request, new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.FORBIDDEN));
		}

		try {
			String fileName = request.getUri();
			if (fileName.equals("/")) fileName = Config.INDEX_PAGE;
			
			FileResponse file = WebServer.instance().getFileManager().fetch(fileName);
			HttpResponse resp = ResponseBuilder.buildFileResponse(file);
			sendResponse(context, request, resp);

		} catch (Exception e) {
			sendResponse(context, request, new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.NOT_FOUND));
			return;
		}
	}
	
	/**
	 * Sends a response.
	 * @param contex The context.
	 * @param request The request.
	 * @param response The response to send.
	 */
	public void sendResponse(ChannelHandlerContext context, HttpRequest request, HttpResponse response) {
		if (response.getStatus() != HttpResponseStatus.OK) {
			response.setContent(ChannelBuffers.copiedBuffer(response.getStatus().toString(), CharsetUtil.UTF_8));
		}
		ChannelFuture future = context.getChannel().write(response);
		future.addListener(ChannelFutureListener.CLOSE);
	}

}
